'use strict';

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

require('better-log/install');

var _TraversalUtils = require('./TraversalUtils');

var _typeAnnotationToJsdocType = require('./typeAnnotationToJsdocType');

var _typeAnnotationToJsdocType2 = _interopRequireDefault(_typeAnnotationToJsdocType);

require('babel/polyfill');

module.exports = function (_ref) {
    var Plugin = _ref.Plugin;
    var t = _ref.types;

    return new Plugin('jsdoc', {
        visitor: {
            'ClassDeclaration|ClassExpression': function ClassDeclarationClassExpression(node, parent, scope, file) {
                _TraversalUtils.addTag.call(this, {
                    title: 'class',
                    name: node.id.name
                });
            },

            'MethodDefinition': function MethodDefinition(node, parent, scope, file) {
                if (scope.block && scope.block.id && scope.block.id.name) {
                    if (node.kind !== 'constructor') {
                        _TraversalUtils.addTag.call(this, 'memberof', {
                            name: scope.block.id.name
                        });
                        if (node['static']) {
                            _TraversalUtils.addTag.call(this, 'static');
                        } else {
                            _TraversalUtils.addTag.call(this, 'instance');
                        }
                    }
                    if (node.key.name) {
                        if (node.kind === 'method') {
                            _TraversalUtils.addTag.call(this, 'method', {
                                name: node.key.name
                            });
                        }
                    }
                }
            },

            'FunctionDeclaration|FunctionExpression': function FunctionDeclarationFunctionExpression(node, parent, scope, file) {
                var path = this;

                if (t.isMethodDefinition(parent)) {
                    path = this.parentPath;
                    if (parent.kind === 'constructor') {
                        path = path.parentPath.parentPath;
                    }
                }

                if (!t.isMethodDefinition(parent)) {
                    _TraversalUtils.addTag.call(path, 'function', {
                        name: node.key && node.key.name
                    });
                }

                if (node.params) {
                    node.params.forEach(function (param) {
                        if (param.type === 'RestElement') {
                            _TraversalUtils.addTag.call(path, {
                                title: 'param',
                                name: param.argument.name,
                                type: '...*'
                            });
                        } else if (param.type === 'AssignmentPattern') {
                            var jsdocType = (0, _typeAnnotationToJsdocType2['default'])(param.left.typeAnnotation && param.left.typeAnnotation.typeAnnotation);
                            _TraversalUtils.addTag.call(path, {
                                title: 'param',
                                name: '[' + param.left.name + '=' + param.right.value + ']',
                                type: jsdocType
                            });
                        } else {
                            var jsdocType = (0, _typeAnnotationToJsdocType2['default'])(param.typeAnnotation && param.typeAnnotation.typeAnnotation);
                            _TraversalUtils.addTag.call(path, {
                                title: 'param',
                                name: param.optional ? '[' + param.name + ']' : param.name,
                                type: jsdocType
                            });
                        }
                    });
                }

                if (parent.kind === 'get') {
                    _TraversalUtils.addTag.call(path, 'member', {
                        name: parent.key.name,
                        type: node.returnType && (0, _typeAnnotationToJsdocType2['default'])(node.returnType.typeAnnotation)
                    });
                } else if (node.returnType) {
                    _TraversalUtils.addTag.call(path, {
                        title: 'returns',
                        type: (0, _typeAnnotationToJsdocType2['default'])(node.returnType.typeAnnotation)
                    });
                }
            }
        }
    });
};